"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.handler = exports.help = exports.commandNames = exports.cliOptionsTypes = exports.rcOptionsTypes = void 0;
const cli_utils_1 = require("@pnpm/cli-utils");
const common_cli_options_help_1 = require("@pnpm/common-cli-options-help");
const dedupe_check_1 = require("@pnpm/dedupe.check");
const plugin_commands_env_1 = require("@pnpm/plugin-commands-env");
const render_help_1 = __importDefault(require("render-help"));
const install_1 = require("./install");
const installDeps_1 = require("./installDeps");
const omit_1 = __importDefault(require("ramda/src/omit"));
// In general, the "pnpm dedupe" command should use .npmrc options that "pnpm install" would also accept.
function rcOptionsTypes() {
    // Some options on pnpm install (like --frozen-lockfile) don't make sense on pnpm dedupe.
    return (0, omit_1.default)(['frozen-lockfile'], (0, install_1.rcOptionsTypes)());
}
exports.rcOptionsTypes = rcOptionsTypes;
function cliOptionsTypes() {
    return {
        ...rcOptionsTypes(),
        check: Boolean,
    };
}
exports.cliOptionsTypes = cliOptionsTypes;
exports.commandNames = ['dedupe'];
function help() {
    return (0, render_help_1.default)({
        description: 'Perform an install removing older dependencies in the lockfile if a newer version can be used.',
        descriptionLists: [
            {
                title: 'Options',
                list: [
                    ...common_cli_options_help_1.UNIVERSAL_OPTIONS,
                    {
                        description: 'Check if running dedupe would result in changes without installing packages or editing the lockfile. Exits with a non-zero status code if changes are possible.',
                        name: '--check',
                    },
                    common_cli_options_help_1.OPTIONS.ignoreScripts,
                    common_cli_options_help_1.OPTIONS.offline,
                    common_cli_options_help_1.OPTIONS.preferOffline,
                    common_cli_options_help_1.OPTIONS.storeDir,
                    common_cli_options_help_1.OPTIONS.virtualStoreDir,
                    common_cli_options_help_1.OPTIONS.globalDir,
                ],
            },
        ],
        url: (0, cli_utils_1.docsUrl)('dedupe'),
        usages: ['pnpm dedupe'],
    });
}
exports.help = help;
async function handler(opts) {
    const include = {
        dependencies: opts.production !== false,
        devDependencies: opts.dev !== false,
        optionalDependencies: opts.optional !== false,
    };
    return (0, installDeps_1.installDeps)({
        ...opts,
        dedupe: true,
        include,
        includeDirect: include,
        lockfileCheck: opts.check ? dedupe_check_1.dedupeDiffCheck : undefined,
        prepareExecutionEnv: plugin_commands_env_1.prepareExecutionEnv.bind(null, opts),
    }, []);
}
exports.handler = handler;
//# sourceMappingURL=dedupe.js.map