"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.handler = exports.help = exports.commandNames = exports.cliOptionsTypes = exports.rcOptionsTypes = void 0;
const path_1 = __importDefault(require("path"));
const cli_utils_1 = require("@pnpm/cli-utils");
const common_cli_options_help_1 = require("@pnpm/common-cli-options-help");
const config_1 = require("@pnpm/config");
const error_1 = require("@pnpm/error");
const find_workspace_dir_1 = require("@pnpm/find-workspace-dir");
const get_context_1 = require("@pnpm/get-context");
const workspace_find_packages_1 = require("@pnpm/workspace.find-packages");
const store_connection_manager_1 = require("@pnpm/store-connection-manager");
const core_1 = require("@pnpm/core");
const logger_1 = require("@pnpm/logger");
const p_limit_1 = __importDefault(require("p-limit"));
const path_absolute_1 = __importDefault(require("path-absolute"));
const pick_1 = __importDefault(require("ramda/src/pick"));
const partition_1 = __importDefault(require("ramda/src/partition"));
const render_help_1 = __importDefault(require("render-help"));
const installCommand = __importStar(require("./install"));
const getSaveType_1 = require("./getSaveType");
// @ts-expect-error
const isWindows = process.platform === 'win32' || global['FAKE_WINDOWS'];
const isFilespec = isWindows ? /^(?:[.]|~[/]|[/\\]|[a-zA-Z]:)/ : /^(?:[.]|~[/]|[/]|[a-zA-Z]:)/;
const installLimit = (0, p_limit_1.default)(4);
exports.rcOptionsTypes = cliOptionsTypes;
function cliOptionsTypes() {
    return (0, pick_1.default)([
        'global-dir',
        'global',
        'only',
        'package-import-method',
        'production',
        'registry',
        'reporter',
        'save-dev',
        'save-exact',
        'save-optional',
        'save-prefix',
        'unsafe-perm',
    ], config_1.types);
}
exports.cliOptionsTypes = cliOptionsTypes;
exports.commandNames = ['link', 'ln'];
function help() {
    return (0, render_help_1.default)({
        aliases: ['ln'],
        descriptionLists: [
            {
                title: 'Options',
                list: [
                    ...common_cli_options_help_1.UNIVERSAL_OPTIONS,
                    {
                        description: 'Link package to/from global node_modules',
                        name: '--global',
                        shortAlias: '-g',
                    },
                ],
            },
        ],
        url: (0, cli_utils_1.docsUrl)('link'),
        usages: [
            'pnpm link <dir>',
            'pnpm link --global (in package dir)',
            'pnpm link --global <pkg>',
        ],
    });
}
exports.help = help;
async function checkPeerDeps(linkCwdDir, opts) {
    const { manifest } = await (0, cli_utils_1.tryReadProjectManifest)(linkCwdDir, opts);
    if (manifest?.peerDependencies && Object.keys(manifest.peerDependencies).length > 0) {
        const packageName = manifest.name ?? path_1.default.basename(linkCwdDir); // Assuming the name property exists in newManifest
        const peerDeps = Object.entries(manifest.peerDependencies)
            .map(([key, value]) => `  - ${key}@${value}`)
            .join(', ');
        logger_1.logger.warn({
            message: `The package ${packageName}, which you have just pnpm linked, has the following peerDependencies specified in its package.json:

${peerDeps}

The linked in dependency will not resolve the peer dependencies from the target node_modules.
This might cause issues in your project. To resolve this, you may use the "file:" protocol to reference the local dependency.`,
            prefix: opts.dir,
        });
    }
}
async function handler(opts, params) {
    const cwd = process.cwd();
    const storeControllerCache = new Map();
    let workspacePackagesArr;
    let workspacePackages;
    if (opts.workspaceDir) {
        workspacePackagesArr = await (0, workspace_find_packages_1.findWorkspacePackages)(opts.workspaceDir, {
            ...opts,
            patterns: opts.workspacePackagePatterns,
        });
        workspacePackages = (0, get_context_1.arrayOfWorkspacePackagesToMap)(workspacePackagesArr);
    }
    else {
        workspacePackages = new Map();
    }
    const store = await (0, store_connection_manager_1.createOrConnectStoreControllerCached)(storeControllerCache, opts);
    const linkOpts = Object.assign(opts, {
        storeController: store.ctrl,
        storeDir: store.dir,
        targetDependenciesField: (0, getSaveType_1.getSaveType)(opts),
        workspacePackages,
    });
    const linkCwdDir = opts.cliOptions?.dir && opts.cliOptions?.global ? path_1.default.resolve(opts.cliOptions.dir) : cwd;
    // pnpm link
    if ((params == null) || (params.length === 0)) {
        if (path_1.default.relative(linkOpts.dir, cwd) === '') {
            throw new error_1.PnpmError('LINK_BAD_PARAMS', 'You must provide a parameter');
        }
        await checkPeerDeps(linkCwdDir, opts);
        const { manifest, writeProjectManifest } = await (0, cli_utils_1.tryReadProjectManifest)(opts.dir, opts);
        const newManifest = await (0, core_1.addDependenciesToPackage)(manifest ?? {}, [`link:${linkCwdDir}`], linkOpts);
        await writeProjectManifest(newManifest);
        return;
    }
    const [pkgPaths, pkgNames] = (0, partition_1.default)((inp) => isFilespec.test(inp), params);
    await Promise.all(pkgPaths.map(async (dir) => installLimit(async () => {
        const s = await (0, store_connection_manager_1.createOrConnectStoreControllerCached)(storeControllerCache, opts);
        const config = await (0, cli_utils_1.getConfig)({ ...opts.cliOptions, dir }, {
            excludeReporter: true,
            rcOptionsTypes: installCommand.rcOptionsTypes(),
            workspaceDir: await (0, find_workspace_dir_1.findWorkspaceDir)(dir),
        });
        await (0, core_1.install)(await (0, cli_utils_1.readProjectManifestOnly)(dir, opts), {
            ...config,
            ...(0, config_1.getOptionsFromRootManifest)(config.rootProjectManifestDir, config.rootProjectManifest ?? {}),
            include: {
                dependencies: config.production !== false,
                devDependencies: config.dev !== false,
                optionalDependencies: config.optional !== false,
            },
            storeController: s.ctrl,
            storeDir: s.dir,
            workspacePackages,
        });
    })));
    if (pkgNames.length > 0) {
        let globalPkgNames;
        if (opts.workspaceDir) {
            workspacePackagesArr = await (0, workspace_find_packages_1.findWorkspacePackages)(opts.workspaceDir, {
                ...opts,
                patterns: opts.workspacePackagePatterns,
            });
            const pkgsFoundInWorkspace = workspacePackagesArr
                .filter(({ manifest }) => manifest.name && pkgNames.includes(manifest.name));
            pkgsFoundInWorkspace.forEach((pkgFromWorkspace) => pkgPaths.push(pkgFromWorkspace.rootDir));
            if ((pkgsFoundInWorkspace.length > 0) && !linkOpts.targetDependenciesField) {
                linkOpts.targetDependenciesField = 'dependencies';
            }
            globalPkgNames = pkgNames.filter((pkgName) => !pkgsFoundInWorkspace.some((pkgFromWorkspace) => pkgFromWorkspace.manifest.name === pkgName));
        }
        else {
            globalPkgNames = pkgNames;
        }
        const globalPkgPath = (0, path_absolute_1.default)(opts.dir);
        globalPkgNames.forEach((pkgName) => pkgPaths.push(path_1.default.join(globalPkgPath, 'node_modules', pkgName)));
    }
    const { manifest, writeProjectManifest } = await (0, cli_utils_1.readProjectManifest)(linkCwdDir, opts);
    await Promise.all(pkgPaths.map(async (dir) => {
        await checkPeerDeps(dir, opts);
    }));
    const linkConfig = await (0, cli_utils_1.getConfig)({ ...opts.cliOptions, dir: cwd }, {
        excludeReporter: true,
        rcOptionsTypes: installCommand.rcOptionsTypes(),
        workspaceDir: await (0, find_workspace_dir_1.findWorkspaceDir)(cwd),
    });
    const storeL = await (0, store_connection_manager_1.createOrConnectStoreControllerCached)(storeControllerCache, linkConfig);
    const newManifest = await (0, core_1.link)(pkgPaths, path_1.default.join(linkCwdDir, 'node_modules'), {
        ...linkConfig,
        targetDependenciesField: linkOpts.targetDependenciesField,
        storeController: storeL.ctrl,
        storeDir: storeL.dir,
        manifest,
    });
    if (!opts.cliOptions?.global) {
        await writeProjectManifest(newManifest);
    }
    await Promise.all(Array.from(storeControllerCache.values())
        .map(async (storeControllerPromise) => {
        const storeControllerHolder = await storeControllerPromise;
        await storeControllerHolder.ctrl.close();
    }));
}
exports.handler = handler;
//# sourceMappingURL=link.js.map